`lfdr` <-
function(x, precision=0.01, M=4, nL=20, nR=20){

  x = x[!is.na(x)]
  if(!precision %in% c(0.01, 0.001, 0.0001)){
    stop("precision must be one of 0.01, 0.001, or 0.0001.")
  }

  x = round(x, round(-log10(precision)))
  den = density(x, n=2^16)
  
  mu = round(den$x[which.max(den$y)], -log10(precision))
  mx = round(median(x), round(-log10(precision)))
  
  if(mu > mx){
    xstart = round(quantile(x, 0.20),round(-log10(precision))) - precision*nL
    xend   = mx + precision*nR
    x0 = seq(xstart, xend, by=precision)
    nn = length(x0)
    y0 = numeric(nn)
    for(i in 1:nn){
      y0[i] = den$y[which.min(abs(den$x - x0[i]))]
    }    
    
    data(savgol.coef)
    sa1 = savgol.coef[[1]]
    scoef1 = sa1[sa1$M==M & sa1$nL==nL & sa1$nR==nR,]
    sa2 = savgol.coef[[2]]
    scoef2 = sa2[sa2$M==M & sa2$nL==nL & sa2$nR==nR,]
        
    if(nrow(scoef1) != 1 || nrow(scoef2) != 1){
      stop("not exact one row found in savgol.coef\n")
    }
    
    scoef1 = scoef1[-(1:3)]
    scoef1 = as.numeric(scoef1[which(!is.na(scoef1))])

    scoef2 = scoef2[-(1:3)]
    scoef2 = as.numeric(scoef2[which(!is.na(scoef2))])

    dy1 = dy2 = rep(NA, nn)
    n0  = (nL+1):(nn-nR)
    
    dyt1 = dyt2 = numeric(nn - nL - nR)                
    nk = 0
    for(n in ((-nL):nR)){
      nk = nk + 1
      n1 = n0 + n
      dyt1 = dyt1 + scoef1[nk]*y0[n1]
      dyt2 = dyt2 + scoef2[nk]*y0[n1]
    }
    
    dyt1 = dyt1/precision   
    dy1[n0] = dyt1
    dyt2 = dyt2/precision   
    dy2[n0] = dyt2
    
    found = 0
    cuts0 = c(0.01, 0.005, 0.001, 0.0005, 0.0001)
    for(cut0 in cuts0){        
      kp = which(abs(dy1) < cut0 & dy2 < 0)
      kp = sort(kp)
      if(length(kp)==1){
        mu = x0[kp]
        found = 1
      } else if(length(kp)>1 && all(diff(kp)==1)){
        kp = kp[which.min(abs(dy1[kp]))]
        mu = x0[kp]
        found = 1
      }
      if(found==1){ break }
    }
    
    if(found == 0){
      warstr = "cannot identify the mode of the null distribution,"
      warstr = paste(warstr, "use a conservative estimation\n")
      warning(warstr, immediate.=TRUE)
      
      rx = range(x)
      xx = seq(rx[1], rx[2], by=precision)
      yy = numeric(length(xx))
      for(i in 1:length(xx)){
        yy[i] = den$y[which.min(abs(den$x - xx[i]))]
      }    
      
      which.median = which.min(abs(xx - median(x)))
      which.q1st   = which.min(abs(xx - quantile(x, prob=0.25)))
      
      for(idx in (which.median-1):which.q1st){
        if(2*idx-1 > length(yy)){next}
        mu = xx[idx]
        yt = yy
        yt[(2*idx-1):(idx+1)] = yy[1:(idx-1)]
        if(length(yt) != length(yy)){
          stop("length not match\n")
        }
        if(all(yt < yy + 1e-10)){ break }
      }
    }
    
  }
  
  mu = round(mu, -log10(precision))
  
  y1 = x[x < mu]
  y2 = 2*mu - y1
  y3 = x[x==mu]
  yn = c(y1, y2, y3)

  pi0 = (2*length(y1)+length(y3))/length(x)
  dn0 = density(yn, n = 2^16)
  sigma2 = var(yn)

  from = min(yn)
  to = 2*mu - from

  f0 = fb = lfdr = numeric((to-mu)/precision)
  pt = seq(mu, to, by=precision)
  pt = round(pt, round(-log10(precision)))
  for(i in 1:length(pt)){
    f0[i] = dn0$y[which.min(abs(dn0$x - pt[i]))]
    fb[i] = den$y[which.min(abs(den$x - pt[i]))]
  }

  lfdr = (f0*pi0/fb)

  for(i in 2:length(lfdr)){
    lfdr[i] = min(lfdr[i-1], lfdr[i])
  }

  pl = list(mean0=mu, var0=sigma2, pi0=pi0, precision = precision,
    pt=pt, f0=f0, fb=fb, lfdr=lfdr, median=median(x))
  class(pl) = "lfdr"
  pl
}

