`mergeWindow` <-
function(windowX, wl, probeX, pl, radius, lambda=1/500, window.cut=0.05,
 probe.cut.type = "lfdr", probe.cut=0.1, n.probe=3){

  calculate.peak <- function(pk1, wl, pl, radius, lambda){
    sr = sqrt(pl$var0)
    for(i in 1:nrow(pk1)){
      pkc  = pk1[i,]
      tkp0 = which(pk1$end <= pkc$start & pk1$end >= pkc$start - radius)
      tkp1 = which(pk1$start >= pkc$end & pk1$start <= pkc$end + radius)
      tkp  = c(tkp0, i, tkp1)

      dM = pk1[tkp,]
      pkc$wL = length(tkp0)
      pkc$wR = length(tkp1)
      if(nrow(dM)==1){
        score = dM$probe/sr
      }else{
        ss = get.sd(dM$start, pl$var0, lambda)
        score = mean(dM$probe)/ss
      }
      pkc$x = round(score, -log10(pl$precision))

      if(pkc$x <= wl$mean0){
        lf = 1
      }else{
        idx = (pkc$x - wl$mean0)/wl$precision + 1
        if(idx > length(wl$lfdr)){
          lf = 0
        }else{
          lf = wl$lfdr[idx]
        }
      }
      pkc$wlfdr  = lf
      pk1[i,] = pkc
    }
    pk1
  }

  mpks = character(0)
  pall.lfdr = assign.lfdr(probeX, pl)
  wall.lfdr = assign.lfdr(windowX$x, wl)
  windowX[["wlfdr"]] = wall.lfdr
  windowX[["plfdr"]] = pall.lfdr
  windowX[["probe"]] = probeX
  ww  = which(wall.lfdr <= window.cut)
  pks = windowX[ww,]
  dis = diff(pks$start)
  
  if(nrow(pks) == 0){
    warning("no peak identified\n")
    return(NULL)  
  }
  
  bod = which(dis > 2*radius | pks$chr[-1] != pks$chr[-nrow(pks)] )
  bod = c(0, bod, nrow(pks))

  for(i in 2:length(bod)){
    i1 = bod[i-1]+1
    i2 = bod[i]
    istart = ww[i1] - pks$wL[i1]
    iend = ww[i2] + pks$wR[i2]
    pk1  = windowX[istart:iend,]
    
    # check whether there is enough probes left
    if(probe.cut.type=="lfdr"){
      wpb = which(pk1$plfdr <= probe.cut)
    }else{
      wpb = which(pk1$probe >= probe.cut)
    }
    if(length(wpb) < n.probe){ next }
    
    while(min(pk1$wlfdr, na.rm=TRUE) <= window.cut){
      ## pick up a new peak
      wpk  = which.max(pk1$x)[1]
      pkc  = pk1[wpk,]
      tkp0 = which(pk1$end <= pkc$start & pk1$end >= pkc$start - radius)
      tkp1 = which(pk1$start >= pkc$end & pk1$start <= pkc$end + radius)
      tkp  = c(tkp0, wpk, tkp1)
      if(probe.cut.type=="lfdr"){
        wpb = which(pk1$plfdr[tkp] <= probe.cut)
      }else{
        wpb = which(pk1$probe[tkp] >= probe.cut)
      }
      
      if(length(wpb)>=n.probe){
        chr = unique(as.character(pk1$chr))
        if(length(chr)>1){
          stop("probes in one peak are located on different chromosomes\n")
        }
        ps1 = pk1$start[min(tkp)]
        ps2 = pk1$end[max(tkp)]
        np  = pkc$wL + pkc$wR + 1

        if(np >= n.probe){
          mpks = rbind(mpks, c(chr, ps1, ps2, pkc$x, np, pkc$wlfdr))
        }
      }
      
      pk1 = pk1[-tkp,]
      # check whether there is enough probes left
      if(probe.cut.type=="lfdr"){
        wpb = which(pk1$plfdr <= probe.cut)
      }else{
        wpb = which(pk1$probe >= probe.cut)
      }
      
      if(length(wpb) < n.probe){ break }

      pk1 = calculate.peak(pk1, wl, pl, radius, lambda)
    }
  }
  mpks = data.frame(mpks)
  names(mpks)=c("chr", "start", "end", "score", "n.probe", "lfdr")
  mpks$score = as.numeric(as.character(mpks$score))
  mpks$lfdr = as.numeric(as.character(mpks$lfdr))
  mpks = mpks[order(mpks$lfdr, -mpks$score),]
  mpks
}
