`norm.GC` <-
function(x5, x3, nGC, method=c("lowess", "linear"), f=1/3, n.grp=500){

  norm1 <- function(x5tmp, x3tmp, method, f){
    if(method=="lowess"){
      rr = lowess(x3tmp, x5tmp, f)
      xn1 = x5tmp - rr$y[rank(x3tmp)]
      xn1 = xn1/median(abs(xn1))
    }else if(method=="linear"){
      rr = rq(x5tmp ~ x3tmp)
      xn1 = rr$resid
      xn1 = xn1/median(abs(xn1))
    }
    xn1
  }

  method = method[1]

  tGC = table(nGC)
  uGC = as.numeric(names(tGC))
  odr = order(uGC)
  uGC = uGC[odr]
  tGC = tGC[odr]
  
  w2combine = numeric(0)
  for(i in 1:length(uGC)){
    if(tGC[i] < n.grp){
      w2combine = c(w2combine, i)
    }
  }
  
  wremain = (1:length(uGC))[-w2combine]
  if(length(wremain) < 3){
    stop("no enought observation in each GC group\n")
  }
  min.remain = min(wremain)
  max.remain = max(wremain)
  
  if(any(w2combine > min.remain & w2combine < max.remain)){
    # warning("")
  }
  
  xn = rep(NA, length(x5))
  
  # for the first group
  wtmp  = which(nGC <= uGC[min.remain])
  x5tmp = x5[wtmp]
  x3tmp = x3[wtmp]
  xn[wtmp] = norm1(x5tmp, x3tmp, method, f)
  
  # for the last group
  wtmp  = which(nGC >= uGC[max.remain])
  x5tmp = x5[wtmp]
  x3tmp = x3[wtmp]
  xn[wtmp] = norm1(x5tmp, x3tmp, method, f)

  # for the other groups
  for(kGC in uGC[(min.remain+1):(max.remain-1)]){
    wtmp  = which(nGC == kGC)
    x5tmp = x5[wtmp]
    x3tmp = x3[wtmp]
    xn[wtmp] = norm1(x5tmp, x3tmp, method, f)
  }
  xn
}

