/*
 *  win.c
 *
 *  Created by wei sun on Thu Dec 13 2007.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <R.h>

/**
 * mean value of a vector
 */

double ave(double* x, int j1, int j2){
  int j;
  double av=0.0;
	for(j=j1; j<=j2; j++){
    av += x[j];
	}
	av = av/(j2-j1+1);
	return(av);
}

/**
 * sd of signals within one window
 */

double sd(double* position, int j1, int j2, double vr, double lambda){
  int i, j;
  int nr = j2 - j1 + 1;
  double ss, dij;
  ss = 0.0;
  
	for(i=j1; i<j2; i++){
		for(j=i+1; j<=j2; j++){
			dij = position[j] - position[i];
			ss = ss + exp(-lambda*dij);
		}
	}
	
	ss = sqrt(vr*(nr + 2*ss))/nr;
	return(ss);
}

/**
 * compute window-level signal strength 
 */

void win(double* pL, double* pR, double* x, int* wL, int* wR, double* wx, 
  double* mu0, double* var0, int* dims, double* lambda)
{
	int i, j1, j2;
	int n = dims[0];
	int radius = dims[1];
	double iL, iR, av, ss;
	j1 = j2 = 0;
	
	for(i=0; i<n; i++){
		j1 = j2 = i;
		iL = pL[i];
		iR = pR[i];
		while(j1 >= 0 && pR[j1] >= iL - radius){
			j1--;
		}
		j1++;
		
		while(j2 < n && pL[j2] <= iR + radius){
			j2++;
		}
		j2--;		
		
		wL[i] = i - j1;
		wR[i] = j2 - i;
		
		if(j2==j1){
  		wx[i] = (x[i] - *mu0)/sqrt(*var0);
  	}else{
  		ss = sd(pL, j1, j2, *var0, *lambda);
  		av = ave(x, j1, j2);
  		wx[i] = (av - *mu0)/ss;
  		//Rprintf("i=%d, av=%f, ss=%f, wx[i]=%f\n", i, av, ss, wx[i]);
  	}	
	}
}
