library(mixer)
setwd("~/research/ChIP/demo")

info = read.table("DesignFiles/2005-05-10_HG17Tiling_Set25.ndf", 
  sep="\t", header=TRUE)

info = info[grep("^(chr\\d+|chrX|chrY)$", info$SEQ_ID, perl=TRUE),]
info = info[order(info$SEQ_ID, info$POSITION),]

Cy3 = read.table("PairData/41149_532_pair.txt", sep="\t", header=TRUE)
Cy5 = read.table("PairData/41149_635_pair.txt", sep="\t", header=TRUE)

dim(info)
info[1:2,]
dim(Cy3)
dim(Cy5)
Cy3[1:2,]
Cy5[1:2,]

# ------------------------------------------------------------------ #
# Keep those probes with both information and data 
# ------------------------------------------------------------------ #
dataXY = paste(Cy5$X, Cy5$Y, sep="_")
infoXY = paste(info$X, info$Y, sep="_")
mat    = match(infoXY, dataXY)
wNA    = which(is.na(mat))
if(length(wNA) > 0){
  info = info[-wNA,]
  mat  = mat[-wNA]  
}
Cy5    = Cy5[mat,]
Cy3    = Cy3[mat,]

# ------------------------------------------------------------------ #
# Count the number of GC contents in each sequence 
# ------------------------------------------------------------------ #

seqGC = code.AT.GC(info$PROBE_SEQUENCE, 50)
nGC   = apply(seqGC, 1, sum)
probeX  = norm.GC(log2(Cy5$PM), log2(Cy3$PM), nGC, n.grp=500, method="lowess")

# ------------------------------------------------------------ #
# probe and window level local fdr
# ------------------------------------------------------------ #
pl = lfdr(probeX)

chr = info$SEQ_ID
probe.start = info$POSITION
probe.end   = info$POSITION + 49
radius = 500
lambda = 1/1500

windowX = win(chr, probe.start, probe.end, probeX, pl,
  lambda, radius)

wl = lfdr(windowX$x)

quartz(width=8, height=4)
par(mfrow=c(1,2), mar=c(5,4,2,1))
mp = median(probeX)
plot(pl, mp-7, mp+7, xl="probe-level signal strength")

mw = median(windowX$x)
plot(wl, mw-4, mw+4, xl="window-level signal strength")

# ------------------------------------------------------------ #
# merge peaks
# ------------------------------------------------------------ #

mpks = mergeWindow(windowX, wl, probeX, pl, radius, lambda, 
  window.cut=0.2, probe.cut.type="lfdr",probe.cut=1.0, n.probe=5)

gff(mpks, file="peaks.gff", source="Kim07", feature="41149_pks")


dim(mpks)
mpks[1:2,]
